/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsIDOMNode;

[scriptable, uuid(FBB67442-27A3-483C-8EB2-29C3EED7514C)]
interface inIDOMView : nsISupports
{
  attribute nsIDOMNode rootNode;

  attribute boolean showAnonymousContent;
  attribute boolean showSubDocuments;
  attribute boolean showWhitespaceNodes;
  attribute boolean showAccessibleNodes;
  attribute unsigned long    whatToShow;

  nsIDOMNode getNodeFromRowIndex(in long rowIndex);
  long getRowIndexFromNode(in nsIDOMNode node);

  void rebuild();
};
