/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM ../../../dist/idl/nsIInterAppCommService.idl
 */

#ifndef __gen_nsIInterAppCommService_h__
#define __gen_nsIInterAppCommService_h__


#ifndef __gen_domstubs_h__
#include "domstubs.h"
#endif

#include "jspubtd.h"

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
class nsIURI; /* forward declaration */


/* starting interface:    nsIInterAppCommService */
#define NS_IINTERAPPCOMMSERVICE_IID_STR "7fdd8b68-0b0a-11e3-9b4c-afbc236da250"

#define NS_IINTERAPPCOMMSERVICE_IID \
  {0x7fdd8b68, 0x0b0a, 0x11e3, \
    { 0x9b, 0x4c, 0xaf, 0xbc, 0x23, 0x6d, 0xa2, 0x50 }}

class NS_NO_VTABLE nsIInterAppCommService : public nsISupports {
 public: 

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IINTERAPPCOMMSERVICE_IID)

  /* void registerConnection (in DOMString keyword, in nsIURI handlerPageURI, in nsIURI manifestURI, in DOMString description, in unsigned short appStatus, in jsval rules); */
  NS_IMETHOD RegisterConnection(const nsAString & keyword, nsIURI *handlerPageURI, nsIURI *manifestURI, const nsAString & description, uint16_t appStatus, const JS::Value & rules) = 0;

};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsIInterAppCommService, NS_IINTERAPPCOMMSERVICE_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIINTERAPPCOMMSERVICE \
  NS_IMETHOD RegisterConnection(const nsAString & keyword, nsIURI *handlerPageURI, nsIURI *manifestURI, const nsAString & description, uint16_t appStatus, const JS::Value & rules); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIINTERAPPCOMMSERVICE(_to) \
  NS_IMETHOD RegisterConnection(const nsAString & keyword, nsIURI *handlerPageURI, nsIURI *manifestURI, const nsAString & description, uint16_t appStatus, const JS::Value & rules) { return _to RegisterConnection(keyword, handlerPageURI, manifestURI, description, appStatus, rules); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIINTERAPPCOMMSERVICE(_to) \
  NS_IMETHOD RegisterConnection(const nsAString & keyword, nsIURI *handlerPageURI, nsIURI *manifestURI, const nsAString & description, uint16_t appStatus, const JS::Value & rules) { return !_to ? NS_ERROR_NULL_POINTER : _to->RegisterConnection(keyword, handlerPageURI, manifestURI, description, appStatus, rules); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsInterAppCommService : public nsIInterAppCommService
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIINTERAPPCOMMSERVICE

  nsInterAppCommService();

private:
  ~nsInterAppCommService();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsInterAppCommService, nsIInterAppCommService)

nsInterAppCommService::nsInterAppCommService()
{
  /* member initializers and constructor code */
}

nsInterAppCommService::~nsInterAppCommService()
{
  /* destructor code */
}

/* void registerConnection (in DOMString keyword, in nsIURI handlerPageURI, in nsIURI manifestURI, in DOMString description, in unsigned short appStatus, in jsval rules); */
NS_IMETHODIMP nsInterAppCommService::RegisterConnection(const nsAString & keyword, nsIURI *handlerPageURI, nsIURI *manifestURI, const nsAString & description, uint16_t appStatus, const JS::Value & rules)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIInterAppCommService_h__ */
